@@ -108,6 +108,18 @@ void isolate_punct(char *p)
     snprintf(p,MAX_LINELEN,"%s",buf);
 }
 
+void alphaonly(char *p)
+{
+    char *pp;
+    for(pp=p; *pp; pp++) if(!myisalpha(*pp)) *pp=' ';
+}
+
+void alnumonly(char *p)
+{
+    char *pp;
+    for(pp=p; *pp; pp++) if(!myisalnum(*pp)) *pp=' ';
+}
+
 	/* buffer must have MAX_LINELEN */
 void strfold(char *p)
 {
@@ -117,6 +129,8 @@ void strfold(char *p)
     if(options&op_nocase) for(pp=p; *pp; pp++) *pp=tolower(*pp);
     if(options&op_reaccent) reaccent(p);
     if(options&op_deaccent) deaccent(p);
+    if(options&op_alphaonly) alphaonly(p);
+    if(options&op_alnumonly) alnumonly(p);
     isolate_punct(p);
     if(options&op_nopunct) depunct(p,char_punct);
     if(options&op_nomath) depunct(p,char_math);
