@@ -160,7 +160,7 @@ enum {typ_default, typ_num, typ_func, typ_units, typ_text,
       typ_formal,typ_matrix,typ_vector,typ_set,typ_equation,
       typ_case, typ_nocase, typ_atext, typ_wlist, typ_comp,
       typ_algexp, typ_litexp, typ_menu, typ_coord, typ_fill,
-      typ_raw,
+      typ_raw, typ_symtext,
       typ_java, typ_src, typ_chem
 };
 
@@ -210,6 +210,7 @@ struct {
       {"select",	typ_menu,	"menu"},
       {"set",		typ_set,	"set"},
       {"sigunits",	typ_units,	"sigunits"},
+      {"symtext",	typ_symtext,	"symtext"},
       {"text",		typ_text,	"case"},
       {"textcomp",	typ_comp,	"textcomp"},
       {"unit",		typ_units,	"units"},
@@ -237,14 +238,16 @@ void p_answer(char *p[MAX_PARM])
 	    if(*tt=='=') {
 		for(j=i;j<6;j++) p[j]=p[j+1]; i--;
 		tt=find_word_start(tt+1); *find_word_end(tt)=0;
-		if(isalpha(*tt)) {
-		    k=search_list(anstype,anstype_no,sizeof(anstype[0]),tt);
-		    	/* unknown type is simply ignored */
-		    if(k>=0) {
-			fprintf(outf,"replytype%d=%s\n",
-				answercnt,anstype[k].def);
-			typ=anstype[k].type;
-		    }
+		k=search_list(anstype,anstype_no,sizeof(anstype[0]),tt);
+		    	/* unknown type is now substituted */
+		if(k>=0) {
+		    fprintf(outf,"replytype%d=%s\n",
+			    answercnt,anstype[k].def);
+		    typ=anstype[k].type;
+		}
+		else {
+		    snprintf(nbuf,sizeof(nbuf),"%s",tt); subst(nbuf);
+		    fprintf(outf,"replytype%d=%s\n\n",answercnt,nbuf);
 		}
 	    }
 	    continue;
@@ -443,8 +446,9 @@ void parm(char *p[MAX_PARM], int ptype)
     p[1]=find_word_start(p[1]);
     snprintf(vbuf,sizeof(vbuf),"%s",p[1]); subst(vbuf);
     if(deftag) repsubst(vbuf);
-    if((pp=strparchr(vbuf,'?'))!=NULL) {
+    if((pp=strparchr(vbuf,'?'))!=NULL && pp[1]!='?') {
 	char buf[MAX_LINELEN+1];
+	if(check_compare(vbuf)==0) goto noif;
 	p2=strparchr(pp,':'); *pp++=0; if(p2!=NULL) *p2++=0;
 	snprintf(buf,sizeof(buf),"%s",vbuf);
 	prepcnt=0; parmprep(buf,ptype);
@@ -459,6 +463,7 @@ void parm(char *p[MAX_PARM], int ptype)
 	fprintf(outf,"!endif\n");
     }
     else {
+noif:
 	prepcnt=0; parmprep(vbuf, ptype);
 	putval(vbuf,i,ptype);
     }
@@ -652,8 +657,9 @@ void p_steps(char *p[MAX_PARM])
     snprintf(vbuf,sizeof(vbuf),"%s",find_word_start(p[0])); subst(vbuf);
     strip_trailing_spaces(vbuf);
     if(vbuf[0]==0) return;
-    if((pp=strparchr(vbuf,'?'))!=NULL) {
+    if((pp=strparchr(vbuf,'?'))!=NULL && pp[1]!='?') {
 	char buf[MAX_LINELEN+1];
+	if(check_compare(vbuf)==0) goto noif;
 	p2=strparchr(pp,':'); *pp++=0; if(p2!=NULL) *p2++=0;
 	snprintf(buf,sizeof(buf),"%s",vbuf);
 	prepcnt=0; parmprep(buf,pt_text);
@@ -670,6 +676,7 @@ void p_steps(char *p[MAX_PARM])
 	fprintf(outf,"!endif\n");
     }
     else {
+noif:
 	prepcnt=0; parmprep(vbuf, pt_text);
 	fprintf(outf,"oefsteps=%s \nnextstep=!nosubst %s \n",vbuf,vbuf);
     }
