@@ -127,7 +127,7 @@ void calc_sort(char *p)
 		      MAX_LINELEN-(p2-ordbuf)-1);
 	    p2+=strlen(p2);
 	}
-	setvar("wims_sort_order",ordbuf);
+	force_setvar("wims_sort_order",ordbuf);
     }
     else { /* case of chars */
 	qsort(buf,strlen(buf),1,_char_sorter);
@@ -136,7 +136,7 @@ void calc_sort(char *p)
 	    for(i=total-1;i>=0;i--) *(p+total-i-1)=buf[i];
 	    *(p+total)=0;
 	}
-	else memmove(p,buf,total);
+	else memmove(p,buf,total+1);
     }
     if(t) lines2rows(p);
 }
@@ -150,7 +150,7 @@ void calc_debug(char *p)
 	/* execute a command thru wimslogd */
 void calc_daemon(char *p)
 {
-    if(!trusted_module()) {*p=0; return;}
+    if(!trusted_module() || is_class_module) {*p=0; return;}
     _daemoncmd(p);
 }
 
@@ -176,7 +176,7 @@ void _get_exec_error(char *errorfname, char *cmdname)
 	}
 	p2=getvar("wims_exec_error"); if(p2==NULL) p2="";
 	snprintf(tmplbuf,sizeof(tmplbuf),"%s\nERROR from %s:\n%s",p2,cmdname,obuf);
-	setvar("wims_exec_error",tmplbuf);
+	force_setvar("wims_exec_error",tmplbuf);
     }
 }
 
@@ -275,7 +275,7 @@ void calc_exec(char *p)
 	read_tmp_file(obuf,nbuf);
 	if(obuf[0]) {
 	    snprintf(nbuf,sizeof(nbuf),"wims_exec_out_%d",i);
-	    setvar(nbuf,obuf);
+	    force_setvar(nbuf,obuf);
 	}
     }
     strip_trailing_spaces(p);
@@ -289,10 +289,16 @@ void calc_mexec(char *p)
 {
     char *public_bin;
     if(robot_access) return;
-    if(trusted_module()!=1) {
+    if(trusted_module()!=1 || is_class_module) {
 	module_error("not_trusted"); *p=0; return;
     }
-    public_bin=bin_dir; bin_dir=module_prefix;
+	/* The following is useless, because mexec content is often given
+	 * by variables */
+/*    if(strstr(p,PARENT_DIR_STRING)!=NULL) {
+	setvar("wims_error_data",p);
+	module_error("illegal_fname"); *p=0; return;
+    }
+*/    public_bin=bin_dir; bin_dir=module_prefix;
     exec_is_module=1;
     calc_exec(p); bin_dir=public_bin;
     exec_is_module=0;
@@ -304,7 +310,7 @@ void _calc_exec(char *p, char *arg0, char *arg1, int n)
     char outfname[MAX_FNAME+1], errorfname[MAX_FNAME+1];
 
     if(robot_access) {*p=0; return;}
-    if(!trusted_module()) {
+    if(!trusted_module() || is_class_module) {
 	if(strcasecmp(tmp_debug,"yes")==0)
 	  accessfile(p,"w","%s/%s.cmd",tmp_dir,arg0);
 	abuf[0]="bin/ch..root";
@@ -312,7 +318,11 @@ void _calc_exec(char *p, char *arg0, char *arg1, int n)
 	abuf[4]=NULL;
     }
     else {
-	abuf[0]=arg0; abuf[1]=arg1; abuf[n]=p; abuf[n+1]=NULL;
+/*	if(strstr(p,PARENT_DIR_STRING)!=NULL) {
+	    setvar("wims_error_data",p);
+	    module_error("illegal_fname"); *p=0; return;
+	}
+*/	abuf[0]=arg0; abuf[1]=arg1; abuf[n]=p; abuf[n+1]=NULL;
     }
     mkfname(outfname,"%s/exec.out",tmp_dir);
     mkfname(errorfname,"%s/exec.err",tmp_dir);
@@ -440,17 +450,16 @@ void calc_randdouble(char *p)
 void calc_randfile(char *p)
 {
     char *pp, n[MAX_FNAME+1];
-    int i, j, t;
+    int i, j;
     
-    t=untrust;
     pp=find_word_start(p); *find_word_end(pp)=0;
     mystrncpy(n,pp,sizeof(n));
     i=datafile_recordnum(pp);
     if(i<=0) {  /* zero records */
-	*p=0; untrust=t; return;
+	*p=0; return;
     }
     j=irand(i); datafile_fnd_record(n,j+1,p);
-    untrust=t; return;
+    return;
 }
 
 	/* random char, word, line, item in a string */
@@ -515,7 +524,7 @@ void calc_randperm(char *p)
     if(n<=0) {*p=0; return;}
     if(n==1 && type==0) {
 	strcpy(p,"1"); 
-	shorder: setvar("wims_shuffle_order","1"); return;
+	shorder: force_setvar("wims_shuffle_order","1"); return;
     }
     if(n>MAX_RANDPERM) n=MAX_RANDPERM;
     for(i=0;i<n;i++) table[i]=i;
@@ -544,7 +553,7 @@ void calc_randperm(char *p)
     }
     if(type) mystrncpy(p,buf1,MAX_LINELEN);
     else mystrncpy(p,buf2,MAX_LINELEN);
-    setvar("wims_shuffle_order",buf2);
+    force_setvar("wims_shuffle_order",buf2);
 }
 
 	/* Computes number of lines in the parm. */
@@ -568,10 +577,8 @@ void calc_itemnum(char *p)
 	/* Computes number of records in the datafile p. */
 void calc_recordnum(char *p)
 {
-    int i,t;
-    t=untrust;
+    int i;
     i=datafile_recordnum(p); mystrncpy(p,int2str(i),MAX_LINELEN);
-    untrust=t;
 }
 
 	/* Computes number of words in the parm. */
@@ -682,9 +689,7 @@ void calc_itemof(char *p)
 	/* pick records in datafile */
 void calc_recordof(char *p)
 {
-    int t=untrust;
     _blockof(p,datafile_recordnum,datafile_fnd_record,"\n:","record");
-    untrust=t;
 }
 
 	/* pick words */
@@ -1018,7 +1023,7 @@ void calc_trim(char *p)
 void calc_date(char *p)
 {
     char *p1, *p2, *p3;
-    if(!trusted_module()) {
+    if(!trusted_module() || is_class_module) {
 	if(strstr(p,"..")!=NULL) return;
 	for(p1=find_word_start(p); *p1; p1=find_word_start(p2)) {
 	    p2=find_word_end(p1);
@@ -1038,7 +1043,7 @@ void calc_listfile(char *p)
     char *pp;
     
     	/* only for trusted modules */
-    if(!trusted_module()) {
+    if(!trusted_module() || is_class_module) {
 	module_error("not_trusted"); *p=0; return; 
     }
     	/* security measures. */
@@ -1454,7 +1459,7 @@ void calc_select(char *p)
 	    if(*find_word_start(p1)==0) continue;
 	    for(j=1;j<=lmax;j++) {
 		snprintf(nbuf,sizeof(nbuf),"wims_select_col_%d",j);
-		fnd_item(p1,j,ibuf); setvar(nbuf,ibuf);
+		fnd_item(p1,j,ibuf); force_setvar(nbuf,ibuf);
 	    }
 	    strcpy(ibuf,buf2); if(compare(ibuf,0,0)) {
 		snprintf(bufp,MAX_LINELEN-(bufp-buf),"%s%c",p1,sep);
@@ -1766,7 +1771,7 @@ void _lookup(char *p, char *fname)
 	/* lookup a definition in a definition file */
 void calc_lookup(char *p)
 {
-    char buf2[MAX_LINELEN+1];
+    char buf2[MAX_LINELEN+1], buf3[MAX_FNAME+1];
     char *p2;
     
     p2=wordchr(p,"in"); if(p2==NULL) {abort: *p=0; return;}
@@ -1774,15 +1779,12 @@ void calc_lookup(char *p)
     mystrncpy(buf2,p2,sizeof(buf2)); substit(buf2);
     *find_word_end(buf2)=0;
     if(strstr(buf2,parent_dir_string)!=NULL) {
-	setvar("wims_error_data",buf2);	module_error("illegal_cmd");
+	force_setvar("wims_error_data",buf2); module_error("illegal_cmd");
     }
     if(strncmp(buf2,"bases/",strlen("bases/"))!=0) {
-	if(untrust&6) goto abort;
-	if(strncmp(buf2,module_prefix,strlen(module_prefix))==0 &&
-	   buf2[strlen(module_prefix)]=='/') _lookup(p,buf2);
-	else if(strncmp(buf2,"datamodule/",strlen("datamodule/"))==0) 
-	  _lookup(p,mkfname(NULL,"modules/data/%s",buf2+strlen("datamodule/")));
-	else _lookup(p,mkfname(NULL,"%s/%s",module_prefix,buf2));
+	if(datafile_check(buf2)!=0 || find_module_file(buf2,buf3,0)!=0)
+	  goto abort;
+	_lookup(p,buf3);
     }
     else _lookup(p,buf2); 
 }
@@ -1792,7 +1794,7 @@ void calc_rename(char *p)
 {
     char buf1[MAX_LINELEN+1], buf2[MAX_LINELEN+1];
     char *p1, *ext, *s;
-    int t;
+    int i,t;
     
     if(robot_access || strstr(p,"getfile")!=NULL) return;
     p1=find_word_start(p); *find_word_end(p1)=0;
@@ -1800,7 +1802,7 @@ void calc_rename(char *p)
     if(p1>p) strcpy(p,p1);
     if(strstr(p,parent_dir_string)!=NULL ||
        strncmp(p,"modules/adm/",strlen("modules/adm/"))==0) {
-	badfile: setvar("wims_error_data",p); module_error("illegal_cmd");
+	badfile: force_setvar("wims_error_data",p); module_error("illegal_cmd");
     }
     if(strncmp(p,module_dir,strlen(module_dir))!=0 &&
        strncmp(p,"modules/data/",strlen("modules/data/"))!=0 &&
@@ -1819,7 +1821,8 @@ void calc_rename(char *p)
     mkfname(buf1,"%s/%s",cwdbuf,p);
     mkfname(buf2,"%s/getfile/rename-%u%s",session_prefix,t,ext);
     if(ftest(buf2)>=0) goto rerand;
-    symlink(buf1,buf2); s=getvar("wims_session"); if(s==NULL) return;
+    i=symlink(buf1,buf2);
+    s=getvar("wims_session"); if(s==NULL) return;
     if(good_httpd) snprintf(buf1,sizeof(buf1),
 			    "getfile/rename-%u%s?session=%s", t,ext,s);
     else snprintf(buf1,sizeof(buf1),"%s?cmd=getfile&session=%s&special_parm=rename-%u%s",
@@ -1976,6 +1979,24 @@ void calc_embraced(char *p)
     mystrncpy(buf,p2,sizeof(buf)); substit(buf);
     if(p1>p) strcpy(p,p1); substit(p);
     if(strcmp(p,"randitem")==0) {_embraced(buf,calc_randitem); goto end;}
+    if(strcmp(p,"extract")==0) {
+	p1=strchr(buf,'{'); if(p1!=NULL) {
+	    p2=find_matching(++p1,'}');
+	    if(p2!=NULL) {
+		memmove(buf,p1,p2-p1); buf[p2-p1]=0;
+	    }
+	    else buf[0]=0;
+	}
+	else buf[0]=0;
+	goto end;
+    }
+    if(strcmp(p,"delete")==0) {
+	for(p1=strchr(buf,'{'); p1; p1=strchr(p1,'{')) {
+	    p2=find_matching(p1+1,'}');
+	    if(p2) strcpy(p1,p2+1); else p1++;
+	}
+	goto end;
+    }
     module_error("syntax_error");
     
     end:
