@@ -284,7 +284,7 @@ void _exec_ex(char *p, char *arg0, char *arg1, int n)
     if(robot_access) {
 	*p=0; return;
     }
-    if(!noout || !trusted_module()) {
+    if(!noout || !trusted_module() || is_class_module) {
 	_calc_exec(p,arg0,arg1,n); if(outputing) output0(p);
 	return;
     }
@@ -466,7 +466,7 @@ void exec_title(char *p)
     if(s==NULL || *s==0) {
 	s=getvar("module_title");
 	if(s==NULL || *s==0) return;
-	setvar("wims_title_title",s);
+	force_setvar("wims_title_title",s);
     }
     phtml_put_base(title_page,0);
 }
@@ -636,7 +636,7 @@ void exec_restart(char *p)
     char buf[MAX_LINELEN+1], *rfn, buf2[MAX_LINELEN+1];
 /*    long int t; */
     
-    if(robot_access || outputing || !trusted_module()) return;
+    if(robot_access || outputing || !trusted_module() || is_class_module) return;
 /*    accessfile(buf,"r","%s/restart.time",s2_prefix);
     t=atoi(buf); if(t==nowtime) return;	*/ /* possible looping */
     mystrncpy(buf,find_word_start(p),sizeof(buf));
@@ -826,7 +826,7 @@ void exec_form(char *p)
 	else if(strncasecmp(m,"file",4)==0) {
 	    m="post\" enctype=\"multipart/form-data";
 	    snprintf(abuf,sizeof(abuf),"?form-data%ld%s",random(),a); a=abuf;
-	    setvar("wims_form_method","");
+	    force_setvar("wims_form_method","");
 	}
 	else m=default_form_method;
     }
@@ -837,7 +837,7 @@ void exec_form(char *p)
 	return;
     }
     output("<p><form action=\"%s%s\"%s method=\"%s\" %s>\n%s",ref_name,a,st,m,opt,wn);
-    if(a!=abuf && a[0]) setvar("wims_ref_anchor","");
+    if(a!=abuf && a[0]) force_setvar("wims_ref_anchor","");
     for(i=0;i<follow_no;i++) {
 	if(robot_access && follow_list[i]==ro_session) continue;
 	if(!new && follow_list[i]!=ro_session 
@@ -891,7 +891,7 @@ void exec_setdef(char *p)
 {
     char *p1, *pp;
     char nbuf[MAX_LINELEN+1], fbuf[MAX_LINELEN+1], tbuf[MAX_LINELEN+1];
-    if(robot_access || !trusted_module()) return;
+    if(robot_access || !trusted_module() || is_class_module) return;
     p1=wordchr(p,"in"); if(p1==NULL) module_error("syntax_error");
     *p1=0; p1=find_word_start(p1+strlen("in"));
     strcpy(nbuf,p);
@@ -986,7 +986,7 @@ void exec_getfile(char *p)
     if(*prompt!=0) *prompt++=0;
     prompt=find_word_start(prompt);
     if(*p==0 || !outputing) return;
-    if(!trusted_module()) return;
+    if(!trusted_module() || is_class_module) return;
     s=getvar(ro_name[ro_session]);
     if(s==NULL || *s==0 || strstr(s,"robot")!=NULL) return;
     mystrncpy(url,ref_name,sizeof(url));
@@ -1461,7 +1461,7 @@ void exec_msg(char *p)
     if(*p2) {
 	*p2=0; p2=find_word_start(p2+1);
     }
-    setvar("wims_error",p1); setvar("wims_error_parm",p2);
+    force_setvar("wims_error",p1); force_setvar("wims_error_parm",p2);
     l=getvar("module_language");
     if(l!=NULL && strlen(l)==2) {
 	snprintf(buf,sizeof(buf),"msg.phtml.%s",l);
@@ -1571,7 +1571,7 @@ void exec_mailto(char *p)
 {
     char *p1,*p2,*pp;
 
-    if(!trusted_module()) return;
+    if(!trusted_module() || is_class_module) return;
     p1=strchr(p,'\n'); if(p1==NULL) return;
     *p1++=0; p=find_word_start(p);
     p2=strchr(p1,'\n'); if(p2==NULL) return;
