@@ -25,6 +25,7 @@
 
 	/* Limit for any data working files. */
 int WORKFILE_LIMIT=512*1024;
+int is_class_module=0;
 
 int trusted_module(void);
 void phtml_put_base(char *fname,int cache);
@@ -49,6 +50,10 @@ char *outptr;
   /* These modules can execute private programs.
    * adm/ modules are always trusted, so need no definition here. */
 char *trusted_modules="";
+		/* bit 0: module is not trusted.
+		 * bit 1: file in wimshome.
+		 * bit 2: readdef or file in writable directory.
+		 */
 int untrust=0;  /* non-zero if user detrusts the module. */
 
 int error_status=0;
@@ -57,6 +62,20 @@ char pidbuf[32];
 
 void delete_pid(void);
 
+	/* Internal use only */
+void _debug(char *s,...)
+{
+    va_list vp;
+    char buf[MAX_LINELEN+1];
+
+    va_start(vp,s);
+    vsnprintf(buf,sizeof(buf),s,vp);
+    va_end(vp);
+    setvar("debug",buf); module_error("debug");
+    exit(1);
+}
+
+
 	/* HTTP response header for non-processed CGI interface */
 void nph_header(int code)
 {
@@ -65,6 +84,7 @@ void nph_header(int code)
 	case 200: cstr="OK"; break;
 	case 301: cstr="Moved Permanently"; break;
 	case 302: cstr="Moved Temporarily"; break;
+	case 420: cstr="WIMS Nested Error"; break;
 	case 450: cstr="WIMS User Error"; break;
 	case 500: cstr="WIMS Internal Error"; break;
 	case 550: cstr="WIMS Module Error"; break;	
@@ -133,11 +153,12 @@ void nametoolong(char *p)
 {
     char buf[MAX_FNAME+17];
     snprintf(buf,sizeof(buf),"%s...",p);
-    setvar("wims_error_data",buf);
+    force_setvar("wims_error_data",buf);
     module_error("file_name_too_long");
 }
 
 enum{is_file, is_dir, is_exec, is_fifo, is_socket, is_unknown};
+off_t ftest_size;
 
 	/* A simple front-end of stat(). */
 int ftest(char *fname)
@@ -146,6 +167,7 @@ int ftest(char *fname)
 /* if(fname[0]=='/' || fname[0]=='.') fprintf(stderr,"ftest: %s\n",fname); */
     mystrncpy(lastftest,fname,sizeof(lastftest));
     if(stat(fname,&ftst)) return lastftype=-1;
+    ftest_size=ftst.st_size;
     if(S_ISREG(ftst.st_mode)) {
 	if((ftst.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH))!=0) return lastftype=is_exec;
 	else return lastftype=is_file;
@@ -330,21 +352,26 @@ void free_mcache(void)
 void cleantmpdir(void);
 void user_error_log(char msg[]);
 
+void nested_error(char msg[])
+{
+    fprintf(stderr,"\nNested error! %s\n",msg);
+    nph_header(420);
+    printf("\r\n\r\nWIMS error processing aborted on nested error.\r\n\r\n%s\r\n",msg);
+    delete_pid(); exit(1);
+}
+
 	/* Send an error message to requester and exit.
 	 * This is for user errors, language-sensitive. */
 void user_error(char msg[])
 {
     char erfname[MAX_FNAME+1];
 
-    if(error_status) {
-	fprintf(stderr,"\nNested error!\n");
-	delete_pid(); exit(1);
-    }
+    if(error_status) nested_error(msg);
     error_status=1;
     mkfname(erfname,"%s.%s",user_error_msg_file,lang);
     if(ftest(erfname)!=is_file) internal_error("user_error(): error message file not found.\n\
 Bad installation.");
-    setvar("wims_user_error",msg);
+    force_setvar("wims_user_error",msg);
     if(strcmp(msg,"threshold")!=0) user_error_log(msg); 
     memmove(module_prefix,".",2); 
     if(lastout_file!=-1 && outputing) {
@@ -369,9 +396,7 @@ void module_error(char msg[])
     char *p;
     WORKING_FILE mf;
 
-    if(error_status) {
-	fprintf(stderr,"\nNested error!\n"); goto quit;
-    }
+    if(error_status) nested_error(msg);
     error_status=1; untrust=0;
     module_error_log(msg);
     nph_header(550);
@@ -396,7 +421,7 @@ void module_error(char msg[])
 	else send=0;
     }
     close_working_file(&mf,0);
-    quit: outptr=outbuf; delete_pid(); exit(1);
+    outptr=outbuf; delete_pid(); exit(1);
 }
 
 	/* Output kernel routine */
@@ -493,6 +518,8 @@ int trusted_module(void)
        strcmp(modname,home_module)==0 ||
        memcmp(modname,"help/",strlen("help/"))==0) {
 	tr:
+	if(memcmp(modname,"classes/",strlen("classes/"))==0)
+	  is_class_module=1;
 	setenv("trusted_module","yes",1);
 	return _trusted=1;
     }
@@ -527,7 +554,12 @@ int find_module_file(char *fname, char buf[], int mode)
 	mkfname(buf,"%s%s",getvar("wims_home"),p); goto lastopen;
     }
     if(strcmp(dtest,"writable")==0) {
-	mkfname(buf,"w/%s/%s",module_prefix,fname+strlen("writable/"));
+	if(strncmp(p+1,"TEMP_",5)==0 && strchr(p+1,'/')==NULL) {
+	    mkfname(buf,"%s/%s",tmp_dir,p+1);
+	}
+	else {
+	    mkfname(buf,"w/%s/%s",module_prefix,p+1);
+	}
 	untrust|=4; 
     }
     else {
@@ -535,11 +567,20 @@ int find_module_file(char *fname, char buf[], int mode)
     }
     if(mode) return 0;
     if(ftest(buf)!=is_file) {
-	if(strncmp(fname,"adm/",4)==0 && !trusted_module()) return -1;
+	if(lastftype==is_exec) {
+isexec:
+	    setvar("executable",fname); module_error("executable");
+	    return -1;
+	}
+	if(strncmp(fname,"adm/",4)==0 && 
+	   (!trusted_module() || is_class_module)) return -1;
 	mkfname(buf,"scripts/%s",fname);
 	lastopen:
 	if(mode) return 0;
-	if(ftest(buf)!=is_file) return -1;
+	if(ftest(buf)!=is_file) {
+	    if(lastftype==is_exec) goto isexec;
+	    else return -1;
+	}
     }
     return 0;
 }
@@ -556,7 +597,10 @@ int find_module_file(char *fname, char buf[], int mode)
 
 	/* returns 1 if violation */
 int datafile_check(char *name) {
-    if((untrust&255)==0) return 0; else return 1;
+    if((untrust&255)==0) return 0;
+    if(strncmp(name,"data/",strlen("data/"))==0) return 0;
+    if(strncmp(name,"authors/",strlen("authors/"))==0) return 0;
+    return 1;
 }
 
 	/* returns 0 if success */
@@ -589,10 +633,11 @@ char *_nextdata(char *p)
 unsigned int datafile_recordnum(char *p)
 {
     char nbuf[MAX_LINELEN+1], *pp;
-    int i;
+    int i, t, ret;
 
+    t=untrust; ret=0;
     if(direct_datafile) mystrncpy(nbuf,p,sizeof(nbuf));
-    else if(datafile_check(p)!=0 || find_module_file(p,nbuf,0)) return 0;
+    else if(datafile_check(p)!=0 || find_module_file(p,nbuf,0)) goto ret;
     readdatafile(nbuf); 
     for(i=nextdatacache, pp=datacache[i]; *pp;) {
 	pp=_nextdata(pp+1); i++;
@@ -600,20 +645,23 @@ unsigned int datafile_recordnum(char *p)
 	    datacache[i]=pp; nextdatacache=i;
 	}
     }
-    return i-1;
+    ret=i-1;
+    ret:
+    untrust=t;
+    return ret;
 }
 
 	/* datafile structure: find record n, starting from 1 */
 char *datafile_fnd_record(char *p, int n, char bf[])
 {
     char nbuf[MAX_LINELEN+1], *pp, *p2;
-    int i;
+    int i, t;
 
-    bf[0]=0;
-    if(n<0) return bf;
+    bf[0]=0; t=untrust;
+    if(n<0) goto ret;
     if(direct_datafile) mystrncpy(nbuf,p,sizeof(nbuf));
-    else if(datafile_check(p)!=0 || find_module_file(p,nbuf,0)) return bf;
-    readdatafile(nbuf); if(*lastdata==0) return bf;
+    else if(datafile_check(p)!=0 || find_module_file(p,nbuf,0)) goto ret;
+    readdatafile(nbuf); if(*lastdata==0) goto ret;
     if(n>nextdatacache) {
 	for(i=nextdatacache, pp=datacache[i]; i<n && *pp;) {
 	    pp=_nextdata(pp+1); i++;
@@ -623,7 +671,7 @@ char *datafile_fnd_record(char *p, int n, char bf[])
 	}
     }
     else pp=datacache[n];
-    if(!*pp) return bf;
+    if(!*pp) goto ret;
     if(n>1 || (n==1 && *pp!=tag_string[1])) pp+=strlen(tag_string);
     else if(n==1) pp+=strlen(tag_string)-1;
     if(n<nextdatacache) p2=datacache[n+1];
@@ -636,7 +684,8 @@ char *datafile_fnd_record(char *p, int n, char bf[])
     if(p2-pp>=MAX_LINELEN) p2=pp+MAX_LINELEN-1;
     if(p2<pp) p2=pp;
     memmove(bf,pp,p2-pp); bf[p2-pp]=0;
-    return bf;
+    ret:
+    untrust=t; return bf;
 }
 
 char hex2char(char c1, char c2)
@@ -1175,23 +1224,23 @@ int checkhost(char *hlist)
 	 * The content of bf2[] is destroyed. */
 int checkhostt(char bf2[])
 {
-    char *p1, *p2;
+    char *p1, *p2, *p3;
 	/* compare with starting time */
     for(p1=strchr(bf2,'>'); p1!=NULL; p1=strchr(p1+1,'>')) {
 	if(p1>bf2 && !isspace(*(p1-1))) continue;
-	p1++; p2=find_word_end(p1);
-	if(p2-p1!=14) continue;
-	p1[8]='.'; p1[11]=':'; if(*p2) *p2++=0;
-	if(strncmp(nowstr,p1,14)<0) return 0;
+	p3=find_word_start(++p1); p2=find_word_end(p3);
+	if(p2-p3!=14) continue;
+	p3[8]='.'; p3[11]=':'; if(*p2) *p2++=0;
+	if(strncmp(nowstr,p3,14)<0) return 0;
 	strcpy(p1-1,p2); p1-=2;
     }
 	/* compare with ending time */
     for(p1=strchr(bf2,'<'); p1!=NULL; p1=strchr(p1+1,'<')) {
 	if(p1>bf2 && !isspace(*(p1-1))) continue;
-	p1++; p2=find_word_end(p1);
-	if(p2-p1!=14) continue;
-	p1[8]='.'; p1[11]=':'; if(*p2) *p2++=0;
-	if(strncmp(nowstr,p1,14)>0) return 0;
+	p3=find_word_start(++p1); p2=find_word_end(p3);
+	if(p2-p3!=14) continue;
+	p3[8]='.'; p3[11]=':'; if(*p2) *p2++=0;
+	if(strncmp(nowstr,p3,14)>0) return 0;
 	strcpy(p1-1,p2); p1-=2;
     }
     p1=find_word_start(bf2); if(*p1==0) return 1;
@@ -1219,15 +1268,17 @@ void instex_flush(void)
     wrapexec=0; call_ssh("%s/%s >%s/ins.Out 2>%s/ins.Err",
 			 bin_dir,instex_processor,
 			 tmp_dir,tmp_dir);
-    call_ssh("mv %s %s", instex_fname,s2_prefix);
+    call_ssh("mv %s %s >/dev/null 2>/dev/null", instex_fname,s2_prefix);
     instex_src[0]=instex_fname[0]=0; instex_cnt=0;
 }
 
 	/* put last.phtml */
 void putlastout(void)
 {
+    int t;
     if(instex_cnt>0) instex_flush();
-    catfile(stdout,"%s/%s",s2_prefix,lastout);
+    t=catfile(stdout,"%s/%s",s2_prefix,lastout);
+    if(t==0) printf("Content-type: text/plain\r\n\r\n");
 }
 
 struct sockaddr_un sun;
