@@ -233,7 +233,8 @@ int check_session(void)
 	if(strncmp(tbuf,"QUERY_STRING=",strlen("QUERY_STRING="))==0 && 
 	   strcmp(pp,p1)==0 && strstr(session_prefix,"_test")==NULL) {
 			/* query string does not change */
-	    if(ftest(mkfname(NULL,"%s/%s",s2_prefix,lastout))==is_file) {
+	    if(ftest(mkfname(NULL,"%s/%s",s2_prefix,lastout))==is_file &&
+	       ftest_size > 0) {
 		uselast:
 		putlastout(); delete_pid(); exit(0);
 	    }
@@ -319,6 +320,7 @@ int check_session(void)
 	    else rapidfiredelay=2;
 	}
 	if(ftest(mkfname(fnbuf,"%s/%s",s2_prefix,lastout))==is_file
+	   && ftest_size > 0
 	   && stat(fnbuf,&st) == 0
 	   && st.st_mtime > nowtime-rapidfiredelay && st.st_mtime <= nowtime)
 	  goto uselast;
@@ -601,6 +603,28 @@ void parse_ro_names(void)
       var_proc(NULL,0);
 }
 
+
+	/* returns positive or 0 if var_def found, otherwise returns -1. */
+int var_def_check(char *name)
+{
+    char *p, nbuf[MAX_NAMELEN+1];
+    int i,tt;
+    
+    tt=-1;
+    for(p=name+strlen(name);p>name && myisdigit(*(p-1));p--);
+    if(*p && *p!='0' && p>name) tt=atoi(p);
+    else p=name+strlen(name);
+    if(p>name+MAX_NAMELEN) p=name+MAX_NAMELEN;
+    memmove(nbuf,name,p-name); nbuf[p-name]=0;
+    i=search_list(var_def,defined_var_total,sizeof(var_def[0]),nbuf);
+    if(i<0) return -1;
+    while(i>0 && tt<var_def[i].beg && strcmp(nbuf,var_def[i-1].name)==0) i--;
+    while(i<defined_var_total-1 && tt>var_def[i].end &&
+	      strcmp(nbuf,var_def[i+1].name)==0) i++;
+    if(tt<var_def[i].beg || tt>var_def[i].end) return -1;
+    return i;
+}
+
 int var_def_name(char *n, int v)
 {
     char *q, *r;
@@ -699,21 +723,10 @@ void get_var_defs(void)
 	/* returns 1 if hacked, else 0. */
 int try_hack(char *var)
 {
-    int i, tt, al;
-    char *p;
-    char nbuf[MAX_NAMELEN+1], vbuf[16];
-    snprintf(nbuf,sizeof(nbuf),"%s",var);
-    for(p=nbuf+strlen(nbuf);p>nbuf && myisdigit(*(p-1));p--);
-    if(*p && *p!='0' && p>nbuf) {
-	tt=atoi(p); *p=0;
-    }
-    else tt=-1;
-    i=search_list(var_def,defined_var_total,sizeof(var_def[0]),nbuf);
+    int i, al;
+    char vbuf[16];
+    i=var_def_check(var);
     if(i<0) return 0;
-    while(i>0 && tt<var_def[i].beg && strcmp(nbuf,var_def[i-1].name)==0) i--;
-    while(i<defined_var_total-1 && tt>var_def[i].end &&
-	      strcmp(nbuf,var_def[i+1].name)==0) i++;
-    if(tt<var_def[i].beg || tt>var_def[i].end) return 0;
     al=var_def[i].allow;
     if(al != var_allow_any) switch(cmd_type) {
 	case cmd_new:
@@ -792,6 +805,7 @@ void var_proc(char *fname,int cache)
     char *p, tbuf[MAX_LINELEN+1];
 
     if(fname!=NULL && read_module_file(fname)) return;
+    if(untrust&6) get_var_privileges();
     while(m_file.linepointer<m_file.linecnt) {
 	t=m_file.lines[m_file.linepointer].isstart;
 	if((t&~2)!=1 || m_file.lines[m_file.linepointer].llen==0) {
@@ -825,23 +839,12 @@ void var_deposit(char *p)
 	/* Check and set variables passed in query_string */
 void set_vars_from_parm(void)
 {
-    int i,j,nn,tt,al;
-    char c, *s,*p, vbuf[MAX_LINELEN+1];
+    int i,j,al;
+    char *s, vbuf[MAX_LINELEN+1];
     if(forceresume) return;
     for(i=0; i<user_var_no; i++) {
-	c=0; s=user_variable[i].name; nn=strlen(s);
-	for(p=s+nn;p>s && myisdigit(*(p-1));p--);
-	if(*p && *p!='0' && p>s) {
-	    nn=p-s; tt=atoi(p); c=*p; *p=0;
-	}
-	else tt=-1;
-	j=search_list(var_def,defined_var_total,sizeof(var_def[0]),s);
-	if(j<0) {if(c) *p=c; continue;}	/* Unknown variable: simply ignored. */
-	while(j>0 && tt<var_def[j].beg && strcmp(s,var_def[j-1].name)==0) j--;
-	while(j<defined_var_total-1 && tt>var_def[j].end &&
-	      strcmp(s,var_def[j+1].name)==0) j++;
-	if(c) *p=c;
-	if(tt<var_def[j].beg || tt>var_def[j].end) continue;
+	j=var_def_check(user_variable[i].name);
+	if(j<0) continue;
 	     /* check permissions */
 	al=var_def[j].allow;
 	if(al != var_allow_any) switch(cmd_type) {
@@ -1092,6 +1095,7 @@ void phtml_put(char *fname,int cache)
     outputing=1;
      /* File not found; we give empty output, but no error message. */
     if(fname!=NULL && read_module_file(fname)!=0) return;
+    if(untrust&6) get_var_privileges();
     while(m_file.linepointer<m_file.linecnt) {
 	t=m_file.lines[m_file.linepointer].isstart;
 	if((t&~18)!=1) {m_file.linepointer++; continue;}
