@@ -150,6 +150,8 @@ char examlogf[MAX_FNAME+1];	/* examlog file name */
 char examlogd[MAX_FNAME+1];	/* examlog file name */
 char exam_sheetexo[32];	/* sheet data of an exam */
 char loadavg[64];
+			/* user file variable access control. */
+char *var_readable, *var_writable, *var_nr, *var_nw, *var_pfx;
 int hostcquota;
 int var_noexport;	/* do not export variable */
 
@@ -205,6 +207,7 @@ int coord_input=0;
 	/* special parm used for special cmds (getins, etc). */
 char *ro_name[]={
       "cmd" ,
+      "empty",
       "lang" ,
       "module" ,
       "session" ,
@@ -217,8 +220,8 @@ char *ro_name[]={
       "worksheet"
 };
 enum {
-    ro_cmd, ro_lang, ro_module, ro_session, ro_special_parm, ro_special_parm2,
-      ro_useropts, ro_wsession, ro_subsession, ro_win, ro_worksheet
+    ro_cmd, ro_empty, ro_lang, ro_module, ro_session, ro_special_parm,
+      ro_special_parm2, ro_useropts, ro_wsession, ro_subsession, ro_win, ro_worksheet
 } RO_NAMES;
 #define RO_NAME_NO (sizeof(ro_name)/sizeof(ro_name[0]))
 
@@ -269,10 +272,14 @@ struct {
       {"module_start_time",0},
       {"now",		1},	/* date and time, yyyymmdd.hh:mm:ss */
       {"nowseconds",	1},	/* date and time, seconds since EPOCH */
+      {"nr",		1},	/* non-readable variables in user file, words */
+      {"nw",		1},	/* non-writable variables in user file, words */
       {"otherclass",	1},	/* Remember other logins */
       {"participate",	1},	/* superclass definition */
+      {"prefix",	1},	/* user file prefix */
       {"protocol",	0},	/* http protocol */
       {"rafale",	0},	/* rapidfire request information */
+      {"readable",	1},	/* readable variables in user file, words */
       {"realuser",	1},	/* real user for supervisor in gateway */
       {"req_time",	0},	/* time of the request */
       {"sclassdir",	1},
@@ -289,7 +296,8 @@ struct {
       {"supervisor",	1},	/* real name of the supervisor */
       {"supervisormail",1},	/* email of supervisor */
       {"trustfile",	1},	/* trusted files in special adm modules */
-      {"useropts",	1}	/* user options */
+      {"useropts",	1},	/* user options */
+      {"writable",	1},	/* writable variables in user file, words */
 };
 #define INTERNAL_NAME_NO (sizeof(internal_name)/sizeof(internal_name[0]))
 
@@ -1103,9 +1111,10 @@ int main(int argc, char *argv[], char *envp[])
     initalarm();
     
     executed_gotos=insert_no=output_length=0; ins_alt[0]=0;
+    setvar("empty",""); 	/* lock this variable */
     setvar("wims_version",wims_version);
     setvar("wims_version_date",wims_version_date);
-    setvar("wims_main_font","iso-8859-1");
+    setvar("wims_main_font","utf-8");
     take_httpd_vars();
 
     main_configure();
@@ -1181,7 +1190,9 @@ int main(int argc, char *argv[], char *envp[])
     if(mode==mode_popup && insert_no==0) {
 	p=getvar("wims_mode");
 	if(p!=NULL && strcmp(p,"popup")==0) {
-	    remove_tree(session_prefix); remove_tree(s2_prefix);
+	    remove_tree(session_prefix); 
+	    
+	    remove_tree(s2_prefix);
 	}
     }
     return 0;
